*****************************
*** Introduction to Stata ***
*****************************
* September 6, 2016
* Michele Claibourn
* StatLab: Data Analytics
* University of Virginia
* statlab@virginia.edu
* http://data.library.virginia.edu/statlab/
*****************************

**************
* STATA TIPS *
**************

/* 
Useful Shortcuts: 
	ctrl+9 --> opens a new do-file editor 
	ctrl+N --> opens a new do-file
	ctrl+D --> to submit commands (or click the 'execute' button) 

Syntax Highlighting: 
	blue --> denotes built-in commands/functions
	red --> denotes string text (not commands/functions or other non-numeric information) 
	green --> denotes comments (information not processed by Stata when executed) 
	
Using comments to annotate your do file (three ways):
	1)  use an asterisk (*) at the beginning of a line (single line comment)
	2)  use two forward slashes (//) at the end of any command line (single line comment)
	3)  use two forward slashes and two asterisks (/* */) to bookend a multi-line comment 
	
Three forward slashes (///) will break up any long command into multiple lines.
*/

************************
* STARTING THE DO-FILE *
************************

clear // clears out any existing data from Stata's working memory

version 14
/* Specified the version of Stata you're using.
Stata periodically releases new versions; do-files written for older 
versions might stop working. The version command prevents that. */

cd "C:\Users\mpc8t\Box Sync\mpc\DataServices\Workshops\StataWorkshop\IntroStata2016"
/* Changes the directory to whatever folder is specified. 
This path is specific to my computer.  Update your path to the location where
you saved the workshop materials. You must use the path as it would appear in 
your operating system (Windows, Mac OS X), e.g., on a Mac this might be
cd "/Users/mpc8t/Box Sync/mpc/DataServices/Workshops/StataWorkshop/IntroStata2016") */

* Or click "File --> Change Working Directory" 

capture log close // closes any log file that may have already been open
* capture suppresses the error generated if no log is open 

log using "introStata.log", replace // can use replace or append
/* Logs document your Stata session, including commands issued as well as output 
generated. They can be recorded in two different formats; if no file extension 
is specified, the default is .smcl (Stata Markup and Control Language), this
preserves fonts and colors; I prefer saving as plain text (by specifying either 
.log or .txt). */


***************************
* READING AND SAVING DATA *
***************************

*** Stata Data Files: use
use "collegeCompletion.dta", clear  /* clear removes data currently in memory;
									Stata only allows one dataset at a time to be open */
									
*** Delimited Text Data Files: import delimited 
/* Use this option for ASCII data files, or, text files in which data points are usually delimited 
by commas, spaces, or tabs.  These files are designed to be as portable as possible, so they are not 
dependent on any partcular software, but they lack variable/value labels, and they may require a 
codebook and/or data dictionary to read in.

	1) The first line should have variable names (32 characters or less; spaces, 
	special characters will be removed, except the underscore [_]; the second line 
	begins the data. Stata is case-sensitive, it's good practice to use all lower-case 
	letters in variable names. 
	
	2) No blank rows or columns between data.
	
	3) Missing numeric data should be coded as an empty cell, not a space, dot, or any other 
	non-numeric data. Often, 0, 9, or 99 is used to code missing numeric data; this is fine as long 
	as these are not also valid values (we can recode them to missing once in Stata).
	
	4) Commas in numbers or text are problematic, Stata thinks they are a delimiter and won't read 
	the data properly. You must remove the commas from numeric values before saving the file.
*/

import delimited "collegeCompletion.txt", clear 
* Stata is usually pretty good at figuring out how to automatically read in these data, but...

import delimited "collegeCompletion.txt", delimiter(tab) varnames(1) clear
* ... you can also specify options to make sure it's reading in exactly how you want it to.

* For Stata 12 or earlier versions: insheet using "collegeCompletion.txt", clear 

*** Excel Data Files: 
import excel "collegeCompletion.xlsx", sheet("collegeCompletionInstitutionDet") firstrow clear

*** Saving as Stata data file (.dta extension)
save "collegeCompletion.dta", replace 


**********************************
*** DATA MANAGEMENT & CLEANING ***
**********************************

*** Examining & Sorting Data
describe 
browse // read-only spreadsheet
edit // can enter/edit data by hand, but DON'T!
/* Note that a period (.) indicates missing data.  Stata treats these as very large 
numbers, so be careful when using logical operators like greater than (>). */ 

list chronname state // lists selected variables in results window
/* That returned a lot of results! 
Use: 
	enter -- to advance one line at a time
	spacebar -- to advance one page at a time
	q (or break button) -- to quit
When commands are submitted in batch via a do-file, this is what -set more off- gets rid of.
*/

list chronname state level control in 1/10 // list selected variables for observations 1 to 10

* Subset data: using 'if' (by any combination of relational and logical operators):
list chronname state level control if flagship=="X"
list chronname state control if grad_100_value==0 & level=="4-year"

/* YOUR TURN: Which institutions in Virginia have an on-time graduation rate of 0? */


* Sort data in ascending order: sort
sort unitid // sorting by initial id
sort student_count // by number of students
list chronname state control level student_count in 1/10 // lists 10 schools with fewest students

* Sort data in descending order: gsort
gsort -aid_value // add - to variable to indicate descending
list chronname state control level aid_value in 1/10 // lists top 10 in aid value per student


*** Summary Statistics and Frequencies:
* Summary statistics (means, standard deviations, min and max values): summarize
summ grad_150_value // note the abbreviated command
summ grad_150_value grad_100_value student_count fte_value med_sat_value endow_value ft_fac_value
summ endow_value if control=="Public" & level=="4-year" // summary of subset

* Which public universities have the biggest endowments per student? *
list chronname endow_value if control=="Public" & level=="4-year" & endow_value>100000 & endow_value!=.

/* YOUR TURN: What's the average percent of full-time faculty (ft_fac_value) in Public institutions? */


* Frequency distributions: tabulate
tab control // note the abbreviated command
tab level
tab1 control level // -tab1- for sequential frequency tables for multiple variables
tab control level // cross-tabs to view the relationship between two categorical/ordered variables
tab control level, col // include the column percentage option

/* YOUR TURN: How many historically black colleges or universities (hbcu) are in this data? */


*** Variable Manipulation/Generation 
* Change a variable from string to categorical: encode
encode control, gen(controlcat)
br control controlcat // br is abbreviation for browse

* Rename variables: rename
rename grad_150_value grad150
rename grad_100_value grad100
/* Best practices for variable names: keep them short, descriptive, and reasonably 
distinct. If you have a set of related variables (e.g., part of a scale), consider 
using the same prefix or suffix for these. */

* Recode variables: recode
recode grad100 (0/33.3=1) (33.4/66.7=2) (66.8/100=3), gen(gradrate100)
recode grad100 (0/33.3=1) (33.4/66.7=2) (66.8/100=3), into(gradrate150)
tab1 gradrate100 gradrate150
/* The gen() option preserves the original variable and recodes it into a new variable; 
into() is a synonym and does exactly the same thing. Without one of these 
two options, the variables will be recoded so as to overwrite the originals. */

* Create new variables: generate
generate id = _n // generate an id number

gen lnaid = ln(aid_value) // as a mathematical function of an existing variable
summ aid_value lnaid
/* See -help math functions- for more operations that you can perform. */

gen highgradrate = (gradrate100==3) // as a logical function of an existing variable
tab1 gradrate100 highgradrate
/* Notice the difference between using one equals sign (=) for arithmetic 
operations and two (==) for logical operations (which often follow -if-). */

tab basic
gen research = 1 if basic=="Research Universities--high research activity" | /// 
	basic=="Research Universities--very high research activity" | ///
	basic=="Doctoral and Research Universities" 
replace research=0 if research==.
tab research
/* Using three forward slashes is a useful way to break up long commands; three slashes 
tells Stata that the command continues on the next line. */

gen hbcu1=1 if hbcu=="X" // another as a logical function of an existing variable
replace hbcu1=0 if hbcu==""
tab1 hbcu hbcu1

* Create new variables via more sophisticated transformations: egen ('EGEN' = 'Extensions to GENerate')
egen stdaid = std(aid_value) // standardized values of aid_value

* Select variables to keep: drop, keep
drop vsa_* // the asterisk means "all" -- i.e., drop all variables that start with "vsa_"
/* Since the "vsa_" variables are all next to each other in the data file, I could have also 
written the command as: -drop vsa_year-vsa_enroll_elsewhere_after6_tran- (where vsa_year is 
the first variable, vsa_enroll_elsewhere_after6_tran is the last variable, and the middle dash 
captures every variable in between). */

/* 'keep' works the same as 'drop' except you tell Stata to drop everything 
EXCEPT the variables specified. */


*** Variable and Value Labels
* Add variable labels: label variable
label variable research "Carnegie Classification-Research"
la var grad100 "Percent of first-time, full-time undergrads completing degree on time" 
la var level "2-year or 4-year institution"
la var control "Control of Institution"
describe research grad100

* Add value labeles: label define, label values
label define gradrate 1 "less than one-third" 2 "one-third to two-thirds" 3 "more than two-thirds"
label values gradrate100 gradrate150 gradrate
tab1 gradrate100 gradrate150
/* 'label drop' will remove value labels */

la var hbcu1 "Historically Black Institution"
la de yn 1 "yes" 0 "no"
la val hbcu1 yn
describe hbcu1
tab hbcu1

/* Once variable labels are applied, you can use the option -nolabel- with a variety of 
commands in order to display numerical values instead of words in the output. */

* Label the data set: label data
label data "College Completion Data obtained from The Chronical of Higher Education"
describe 

* Add notes to data: notes
notes: This data was downloaded from http://collegecompletion.chronicle.com/
describe
notes
save collegeCompletion_analysis.dta, replace 

/* Notice that I'm preserving the original data by saving my manipulated data using a 
different name. Also in the interest of good workflow practices, I'm using the logic 
of dual workflow to separate data management from analysis. I start my analyses in the 
next section by reading in my cleaned data (for larger projects, these might be two or 
more separate do-files). */


*****************************************
*** DESCRIPTIVE STATISTICS & ANALYSES ***
*****************************************

use collegeCompletion_analysis.dta, replace

*** Review: 'tabulate' and 'summarize' (with expanded options/details)
* For continuous variables: summarize 
summ aid_value
summ grad150, detail

bysort control: summ aid_value // summarize over subsets
bysort level: summ grad100

summ aid_value
return list // see stored values in memory, which can be used in subsequent calculations
display r(mean)
gen aid_center = aid_value - r(mean) // creates variable of centered values

* For categorical variables: tabulate
tab hbcu1 level, col chi2 // -chi2- option for a chi-squared statistic and p-value

tabulate level, summ(grad100) // one- and two-way tables of summary statistics
/* Compare this example to the command above: -bysort level: summ grad100- ... */

/*  YOUR TURN: What's the on-time graduation rate among institutions of different control types? */


*** Using table to create custom tables
/* The 'table' command is especially useful for creating custom tables. 
The general syntax is as follows:
	-table rowvar colvar, contents(stat1 var1 stat2 var2)-
where stat1 stat2 refer to mean, sd, min, max, etc.
Both the column variable and the second statistics (stat2 var2) are optional.*/

/* What is the average number of undergrads and average percent of undergrads receiving 
Pell grants at each level of graduation rate and each category of control? */ 
table gradrate150 control, contents(mean student_count mean pell_value)

*** T-tests
ttest grad100=grad150 // test difference of means between two continuous variables
ttest grad100, by(research) // test the difference in the mean of one variable within categories of another

*** ANOVA: oneway
oneway grad150 control if research==1
oneway grad150 control if research==1, tab bon // use the original numeric version of the outcome
oneway grad150 control if research==1 & controlcat!=1, tab bon
/* The -tab option produces a summary table of descriptive statistics, and the -bon- option 
(short for Bonferroni) will compute a multiple comparisons test, which calculates pairwise 
comparisons and then adjusts them for multiple comparisons */

*** Regression
regress grad150 i.control if research==1 // can't include string variables
regress grad150 i.controlcat if research==1 // -i.- tells stata this is a categorical variable
regress grad150 i.controlcat if research==1 & controlcat!=1
regress grad150 i.controlcat med_sat_value if research==1 


*****************************
*** INTRODUCTORY GRAPHICS ***
*****************************

/* Stata has powerful graphical facilities. As a consequence the code for creating 
nice graphs can get complicated. We'll be focused on just the basics today, but for 
more on graphics in Stata, download previous StatLab materials from Spring 2015. */

* Distribution of variables: histogram
hist grad150
hist pell_value, frac normal // notice the first graph has been replaced
hist student_count, freq // who's got 250k students?
list chronname student_count if student_count>100000

* Keep multiple graphs open: name option
hist lnaid, name(aid, replace)
hist pell_value, name(pell, replace)

* Graphs open in multiple tabs: set autotabgraphs on
set autotabgraphs on
hist lnaid, name(aid, replace)
hist pell_value, name(pell, replace)

* What graphs are stored in memory? graph dir
graph dir

* Call a graph back from memory: graph display
graph display aid

* Drop graphs from memory: graph drop
graph drop aid
graph drop _all // drops everything

* Box plots to compare the distribution of an interval var across levels of a categorical var: graph box
graph box endow_value, over(control) // -over- option segments the plot by the values of the listed variable
graph box endow_value, over(control) nofill // -nofill- option omits the empty category
gen lnendow=ln(endow_value) // log the outcome variable
graph box lnendow if level=="4-year", over(control) nofill // subset with -if-

* Scatter plot to view the relationship between two interval variables: graph twoway scatter
graph twoway scatter grad150 pell_value if level=="4-year" & control=="Public"
twoway scatter grad150 med_sat_value if level=="4-year" & control=="Public" 
scatter grad150 aid_value if level=="4-year" & control=="Public"  // notice the command abbreviations

/* Among 4-year research universities, do graduation rates vary by average student aid? */
twoway scatter grad150 lnaid if research==1 & controlcat!=1 

/* Let's prettify it!
-Overlay multiple graphs, separated by ||
-Add linear regression line: lift
-Add smoothed locally weighted line: lowess */
twoway scatter grad150 lnaid || lfit grad150 lnaid || lowess grad150 lnaid || if research==1 & controlcat!=1 

/* More prettification!
- Improve the legend: -legend- option (with everything in partentheses as sub-options)
	-order(2 3)- omits the first item in the legend
	-label(# "")- specifies a better label for each relevant item 
-title- to add graph title 
-xtitle- and -ytitle- options to add x-axis and y-axis titles */
twoway scatter grad150 lnaid || lfit grad150 lnaid || lowess grad150 lnaid || if research==1 & controlcat!=1, ///
   legend(order(2 3) label(2 "Regression Line") label(3 "Lowess Line")) title("Completion Rate by Student Aid") ///
   xtitle("Average Student Aid (logged)") ytitle("Completion within 150% of Expected Time")

/* Justy for fun: Does the relationship look the same for public/private universities? */
twoway scatter grad150 lnaid if controlcat==2, msymbol(Oh) mcolor(green) || ///
	scatter grad150 lnaid if controlcat==3, msymbol(Oh) mcolor(blue) || ///
	lfit grad150 lnaid if controlcat==2, lcolor(green) || lfit grad150 lnaid if controlcat==3, lcolor(blue) || ///
	lowess grad150 lnaid if controlcat==2, lcolor(midgreen) || lowess grad150 lnaid if controlcat==3, lcolor(midblue) || ///
	if research==1, ytitle("Completion Rate") xtitle("Average Student Aid (logged)") ///
	title("6-Year Completion Rates by Average Student Aid") subtitle("Across 4-Year Research Universities") ///
	legend(label(1 "Private") label(2 "Public") label(3 "OLS Fit") label(4 "OLS Fit") label(5 "Lowess Curve") label(6 "Lowess Curve"))

log close

save collegeCompletion_analysis_cg.dta, replace 

